@@ -1,7 +1,7 @@
 package Terminal::Identify;
 
 # Load the basic Perl pragmas.
-use 5.0260000;
+use 5.026000;
 use strict;
 use warnings;
 
@@ -15,6 +15,7 @@ our @EXPORT = qw(
     whichtermami
     which_terminal
     identify_terminal
+    terminal_identify
     $OutputFormat
 );
 
@@ -22,66 +23,69 @@ our @EXPORT = qw(
 our @ISA = qw(Exporter);
 
 # Set the package version.
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 # Load the Perl modules.
 use POSIX qw(ttyname);
 
-# Define the directory for Inline.
-#use Inline Config => DIRECTORY => '/tmp/';
+# Set the global output format.
+our $OutputFormat = "";
 
-# Use inline C code.
-#use Inline 'C';
+# Set the global term hash variable.
+our %termhash = ();
 
 # Define the Perl BEGIN block.
 BEGIN {
-    # Check the operating system.
-    if ($^O eq "MSWin32") {
-        # Print message into terminal window.
-        print "This module works not on Windows. Bye.\n";
-        # Exit script with error code 1.
-        exit 1;
-    };
     # Set the subroutine aliases.
     *whichtermami = \&whichterminalami;
     *which_terminal = \&whichterminalami;
     *identify_terminal = \&whichterminalami;
+    *terminal_identify = \&whichterminalami;
 };
 
-# Check if Linux command which exists.
-if (! -x "/bin/which" && ! -x "/usr/bin/which") {
-    die "Linux command 'which' does not exist Bye.\n";
-    # Print message into terminal window.
-    # print "Linux command 'which' does not exist Bye.\n";
-    # Exit script with error code 2.
-    # exit 2;
+# ++++++++++++++++++++++++++++++++
+# Perform some preliminary checks.
+# ++++++++++++++++++++++++++++++++
+
+# Check if the operating system is Windows.
+if ($^O eq "MSWin32") {
+    # Print a message into the terminal window.
+    print "This module works not on Windows. Bye.\n";
+    # Exit the script with error code 1.
+    exit 1;
 };
 
-# Check if grep exists.
+# Check if the Linux command 'which' exists.
+my $exitcode = system("which which > /dev/null 2>&1");
+if ($exitcode != 0) {
+    # Print a message into the terminal window.
+    print "The Linux command 'which' does not exist. Bye.\n";
+    # Exit the script with error code 2.
+    exit 1;
+};
+
+# Check if the Linux command 'grep' exists.
 if (!defined `which grep 2>/dev/null` || `which grep 2>/dev/null` eq "") {
-    die "Linux command 'grep' does not exist Bye.\n";
-    # Print message into terminal window.
-    # print "Linux command 'grep' does not exist Bye.\n";
-    # Exit script with error code 3.
-    # exit 3;
+    # Print a message into the terminal window.
+    print "The Linux command 'grep' does not exist. Bye.\n";
+    # Exit the script with error code 3.
+    exit 3;
 };
 
-# Check if users exists.
+# Check if the Linux command 'users' exists.
 if (!defined `which users 2>/dev/null` || `which users 2>/dev/null` eq "") {
-    die "Linux command 'users' does not exist Bye.\n";
-    # Print message into terminal window.
-    # print "Linux command 'grep' does not exist Bye.\n";
-    # Exit script with error code 4.
-    # exit 4;
+    # Print a message into the terminal window.
+    print "The Linux command 'grep' does not exist. Bye.\n";
+    # Exit the script with error code 4.
+    exit 4;
 };
 
-# Check if ps exists.
+# Check if the Linux command 'ps' exists.
 if (!defined `which ps 2>/dev/null` || `which ps 2>/dev/null` eq "") {
-    die "Linux command 'ps' does not exist Bye.\n";
-    # Print message into terminal window.
-    # print "Linux command 'ps' does not exist Bye.\n";
-    # Exit script with error code 4.
-    # exit 5;
+    # Print a message into the terminal window.
+    print "The Linux command 'ps' does not exist. Bye.\n";
+    # Exit the script with error code 5.
+    exit 5;
 };
 
 # Set some variables.
@@ -92,16 +96,15 @@ our $FN_SHELLS = "/etc/shells";
 if (! -e $FN_PASSWD) {
     die "$!, $FN_PASSWD not exists.\n";
 };
+
 # Check if shells exists.
 if (! -e $FN_SHELLS) {
     die "$!, $FN_SHELLS not exists.\n";
 };
 
-# Set the global output format.
-our $OutputFormat = "";
-
-# Create the dictionary termhash.
-our %termhash = ();
+# ++++++++++++++++++++++++++++++++++++++++++
+# Create a dictionary from __DATA__ section.
+# ++++++++++++++++++++++++++++++++++++++++++
 my $line = "";
 my $key = "";
 my $value = "";
@@ -117,7 +120,7 @@ while (<DATA>) {
         $termhash{$key} = $value;
     };
 };
-    
+
 # ============================================================================ #
 # Subroutine trim                                                              #
 #                                                                              #
@@ -171,19 +174,26 @@ sub search_brackets {
 # Subroutine read_file                                                         #
 #                                                                              #
 # Description:                                                                 #
-# Read a file in one chunk. The retrieved content is stored in a string        #
-# variable.                                                                    #
+# Read the complete content of a file in one chunk. The retrieved content is   #
+# stored in a string variable.                                                 #
 #                                                                              #
-# @argument: $_[0] => $file  Filename      (scalar)                            #
-# @return:   $content        File content  (scalar)                            #
+# @argument: $_[0] => $file  Text filename  (scalar)                           #
+# @return:   $content        File content   (scalar)                           #
 # ============================================================================ #
 sub read_file {
     # Assign the function argument to the local variable.
     my $file = $_[0];
-    # Open a file handler for reading.
+    # Initialise the return variable.
+    my $content = "";
+    # If file not exists, return an empty string.
+    if (! -f $file) {
+        # Return an empty string.
+        return $content;
+    };
+    # Open a file handler for reading the file.
     open(my $fh, "<", $file);
     # Read the complete content from the file.
-    my $content = do {local $/; <$fh>};
+    $content = do {local $/; <$fh>};
     # Close the file handler.
     close($fh);
     # Return the file content.
@@ -655,138 +665,151 @@ sub whichterminalami {
 
 __DATA__
 
-# /etc/terminals: valid terminal emulators
 # Begin terminals
+ajaxterm                => AjaxTerm
 alacritty               => Alacritty
-altyo                   => AltYo 
-aminal                  => Aminal 
-aterm                   => aterm 
+altyo                   => AltYo
+aminal                  => Aminal
+anyterm                 => anyterm
+aterm                   => aterm
 blackbox                => Black Box
-black-screen            => Black Screen 
-boxi                    => Boxi 
-contour                 => Contour Terminal 
-cool-retro-term         => Cool Retro Term 
-deepin-terminal         => Deepin Terminal 
-domterm                 => DomTerm 
-edex-ui                 => eDEX-UI 
-Eterm                   => Eterm 
-extraterm               => Extraterm 
-fbpad                   => Fbpad 
-fbterm                  => Fbterm 
-finalterm               => Final Term 
-flexterm                => Flex Term 
-foot                    => foot 
-gnome-terminal-server   => Gnome Terminal 
-guake                   => Guake Terminal 
-havoc                   => Havoc 
-hyper                   => Hyper 
-iterm2                  => iTerm2 
-kitty                   => kitty 
-konsole                 => Konsole 
-kterm                   => kterm 
-lilyterm                => LilyTerm 
-lxterminal              => LXTerminal 
-macterm                 => MacTerm 
-macwise                 => MacWise 
-mate-terminal           => MATE-Terminal 
-mlterm                  => mlterm 
-mlterm-tiny             => mlterm-tiny 
-mrxvt                   => mrxvt 
-okidk                   => OkiDK 
-pantheon-terminal       => Pantheon Terminal 
+black-screen            => Black Screen
+blinkshell              => Blink Shell
+boxi                    => Boxi
+cicslterm               => cicslterm
+cathode                 => Cathode
+commander               => Commander One
+conemu                  => ConEmu
+contour                 => Contour Terminal
+coolterm                => CoolTerm
+cool-retro-term         => Cool Retro Term
+cutecom                 => CuteCome
+deepin-terminal         => Deepin Terminal
+domterm                 => DomTerm
+dterm                   => dterm
+dtterm                  => dtterm
+edex-ui                 => eDEX-UI
+Eterm                   => Eterm
+extraterm               => Extraterm
+fbpad                   => Fbpad
+fbterm                  => Fbterm
+finalterm               => Final Term
+flexterm                => Flex Term
+foot                    => foot
+gnome-terminal-server   => GNOME Terminal
+gtkterm                 => GtkTerm
+guake                   => Guake Terminal
+havoc                   => Havoc
+hterm                   => hterm
+hyper                   => Hyper
+iterm                   => iTerm
+iterm2                  => iTerm2
+kitty                   => kitty
+konsole                 => Konsole
+kterm                   => kterm
+libiterm                => libiterm
+lilyterm                => LilyTerm
+lterm                   => lterm
+lxterminal              => LXTerminal
+macterm                 => MacTerm
+macwise                 => MacWise
+mate-terminal           => MATE-Terminal
+minicom                 => Minicom
+mlterm                  => mlterm
+mlterm-tiny             => mlterm-tiny
+mrxvt                   => mrxvt
+netterm                 => NetTerm
+okidk                   => OkiDK
+pantheon-terminal       => Pantheon Terminal
+picocom                 => PicoCom
 pterm                   => pterm
-powershell              => PowerShell 
-putty                   => PuTTY 
+powershell              => PowerShell
+powerterm               => PowerTerm
+putty                   => PuTTY
 qterminal               => QTerminal
-rocket                  => Rocket 
-roxterm                 => ROXTerm 
-rxvt                    => rxvt 
-rxvt-unicode            => rxvt-unicode 
-rxvt-unicode-truecolor  => rxvt-unicode-truecolor 
-sakura                  => Sakura 
-screen                  => SCREEN 
-stterm                  => stterm 
-tabby                   => Tabby 
+rocket                  => Rocket
+roxterm                 => ROXTerm
+rxvt                    => rxvt
+rxvt-unicode            => rxvt-unicode
+rxvt-unicode-truecolor  => rxvt-unicode-truecolor
+sakura                  => Sakura
+screen                  => SCREEN
+shellcraft              => ShellCraft
+stjerm                  => stjerm
+stterm                  => stterm
+tabby                   => Tabby
 terminal                => Terminal
-terminalpp              => terminalpp 
-terminator              => Terminator 
-terminix                => Terminix 
+terminalpp              => terminalpp
+terminator              => Terminator
+terminix                => Terminix
 terminology             => Terminology
-terminus                => Terminus 
-termite                 => Termite 
-termit                  => Termit 
-tilda                   => Tilda 
-tilix                   => Tilix 
-tinyterm                => TinyTERM 
-tmux                    => tmux 
-tterm                   => TTerm 
-tym                     => tym 
-upterm                  => Upterm 
-urxvt                   => urxvt 
-uxterm                  => UXTerm 
+terminus                => Terminus
+termite                 => Termite
+termit                  => Termit
+teraterm                => Tera Term
+tilda                   => Tilda
+tilix                   => Tilix
+tinyterm                => TinyTERM
+tmux                    => tmux
+treeterm                => TreeTerm
+tterm                   => TTerm
+tym                     => tym
+upterm                  => Upterm
+urxvt                   => urxvt
+uxterm                  => UXTerm
 warp                    => Warp
-wazaterm                => Wazaterm 
-wezterm                 => WezTerm 
+wazaterm                => Wazaterm
+wezterm                 => WezTerm
 windterm                => WindTerm
-wterm                   => Wterm 
-xfce4-terminal          => Xfce4 Terminal 
-xiki                    => Xiki 
-xiterm+thai             => xiterm+thai 
-xterm                   => XTerm 
-xvt                     => Xvt 
-yakuake                 => Yakuake 
+wterm                   => Wterm
+xfce4-terminal          => Xfce4 Terminal
+xiki                    => Xiki
+xiterm                  => xiterm
+xiterm+thai             => xiterm+thai
+xterm                   => XTerm
+xvt                     => Xvt
+yakuake                 => Yakuake
+yat                     => YAT
 zoc                     => ZOC
 zterm                   => ZTerm
 # End terminals
-# EOF
 
-# /etc/shells: valid login shells
 # Begin shells
-/usr/bin/ash
-/usr/bin/bash
-/usr/bin/csh
-/usr/bin/dash
-/usr/bin/dtksh
-/usr/bin/es
-/usr/bin/esh
-/usr/bin/fish
-/usr/bin/jsh
-/usr/bin/ksh
-/usr/bin/mksh
-/usr/bin/nash
-/usr/bin/oksh
-/usr/bin/pdksh
-/usr/bin/pfcsh
-/usr/bin/pfksh
-/usr/bin/psh
-/usr/bin/psh2
-/usr/bin/rbash
-/usr/bin/rc
-/usr/bin/sash
-/usr/bin/screen
-/usr/bin/scsh
-/usr/bin/sh
-/usr/bin/tcsh
-/usr/bin/tmux
-/usr/bin/zsh
+ash
+bash
+csh
+dash
+dtksh
+es
+esh
+fish
+jsh
+ksh
+ksh88
+ksh93
+mksh
+nash
+oksh
+pdksh
+pfbash
+pfcsh
+pfexec
+pfksh
+pftcsh
+pfsh
+pfzsh
+psh
+psh2
+rbash
+rc
+sash
+screen
+scsh
+sh
+tcsh
+tmux
+zsh
 # End shells
-# EOF
-
-#__C__
-
-#include <stdio.h>
-#include <unistd.h>
-
-#/*
-# * -------------------
-# * Function TermPath()
-# * -------------------
-# */
-#char* TermPath() {
-#  char* tty_name = ttyname(STDIN_FILENO);
-#  char* term_path = (tty_name != NULL) ? tty_name : NULL;
-#  return term_path;
-#};
 
 __END__
 
@@ -1092,12 +1115,10 @@ a backup solution must be integrated. This is a to-do for one of the next versio
 =head1 ERROR CODES
 
   1 = Windows is recognised as OS.
-  * 2 = Linux command 'which' does not exist.
-  * 3 = Linux command 'grep' does not exist.
-  * 4 = Linux command 'users' does not exist.
-  * 5 = Linux command 'ps' does not exist.
-
-* not used yet. will be used after several tests on several OS.
+  2 = Linux command 'which' does not exist.
+  3 = Linux command 'grep' does not exist.
+  4 = Linux command 'users' does not exist.
+  5 = Linux command 'ps' does not exist.
 
 =head1 NOTES
 
